/*
 * Copyright (c) 2004 Ciaran McCreesh <ciaranm at gentoo.org>
 *
 * This file is part of uhinv.
 *
 * uhinv is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * uhinv is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * uhinv; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * \file   src/cpu/cpu_linux.c
 * Linux CPU detection.
 *
 * Handle Linux CPU info via /proc/cpuinfo. Calls the relevant arch routines to
 * do the actual entry parsing.
 */

#include <stdio.h>
#include <string.h>

#include "src/uhinv.h"
#include "src/util/util.h"

#include "cpu_generic.h"
#include "cpu_linux.h"
#include "cpu_linux_arm.h"
#include "cpu_linux_mips.h"
#include "cpu_linux_parisc.h"
#include "cpu_linux_power.h"
#include "cpu_linux_sparc.h"
#include "cpu_linux_x86_64.h"
#include "cpu_linux_x86.h"

/**
 * Get and parse a cpuinfo line into key and value components.
 *
 * Returns whether an understandable result was read. Failure doesn't
 * necessarily indicate EOF; we might have encountered a strange line.
 *
 * \param fh        Filehandle from which to read
 * \param keybuf    Buffer for the key, at least KEYBUF_SIZE big
 * \param valuebuf  Buffer for the value, at least VALUEBUF_SIZE big
 */
    int
get_proc_cpuinfo_entry(fh, keybuf, valuebuf)
    FILE *fh;
    char *keybuf;
    char *valuebuf;
{
    char buf[LINEBUF_SIZE];
    char *delimpos;

    if (!fgets(&buf[0], LINEBUF_SIZE, fh))
        return 0;

    delimpos = strchr(&buf[0], ':');
    if (!delimpos)
        return 0;
    *delimpos = '\0';

    strncpynull(&keybuf[0], &buf[0], KEYBUF_SIZE);
    kill_training_whitespace(&keybuf[0]);

    while (*++delimpos == ' ')
        ;
    strncpynull(&valuebuf[0], delimpos, VALUEBUF_SIZE);
    kill_training_whitespace(&valuebuf[0]);

    return 1;
}

/**
 * Get Linux-style cpu information from /proc/cpuinfo.
 *
 * Needs to know what arch we're on so that we can decide how to parse the
 * cpuinfo file.
 *
 * \param cpu_info    Results go here
 * \param arch        What arch are we on?
 */
    int
get_cpu_linux_info(cpu_info, arch)
    cpu_info_T **cpu_info;
    char *arch;
{
    int success = 0;
    FILE *fh;

    if (!(fh = fopen(arguments.proc_cpuinfo, "r")))
    {
        fprintf(stderr, "Error opening %s!\n", arguments.proc_cpuinfo);
        return 0;
    }

    if (0 == strncasecmp(arch, "sparc", 5))
        success = get_cpu_linux_sparc_info(cpu_info, fh);
    else if (0 == strncasecmp(arch, "x86_64", 6))
        success = get_cpu_linux_x86_64_info(cpu_info, fh);
    else if (0 == strncasecmp(arch, "parisc", 6))
        success = get_cpu_linux_parisc_info(cpu_info, fh);
    else if (0 == strncasecmp(arch, "mips", 4))
        success = get_cpu_linux_mips_info(cpu_info, fh);
    else if (0 == strncasecmp(arch, "ppc", 3))
        success = get_cpu_linux_power_info(cpu_info, fh);
    else if (0 == strncasecmp(arch, "arm", 3))
        success = get_cpu_linux_arm_info(cpu_info, fh);
    /* x86 is quirky, could be i[3456]86 */
    else if ((arch[0] == 'i') && (arch[1] >= '3') && (arch[1] <= '6') &&
            (0 == strcmp(arch + 2, "86")))
        success = get_cpu_linux_x86_info(cpu_info, fh);

    fclose(fh);
    return success;
}

/* vim: set tw=80 sw=4 et : */
