/*
 * Copyright (c) 2004 Ciaran McCreesh <ciaranm at gentoo.org>
 *
 * This file is part of uhinv.
 *
 * uhinv is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * uhinv is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * uhinv; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * \file   src/cpu/cpu_linux_sparc.c
 * Linux/SPARC CPU detection.
 *
 * Includes functions for detecting and parsing Linux/SPARC cpuinfo.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "src/uhinv.h"
#include "src/util/util.h"

#include "cpu_generic.h"
#include "cpu_linux.h"

/**
 * Decode a Linux/SPARC model string.
 *
 * Splits modelstr into name, vendor and speed parts.
 *
 * \param modelstr    Model string to be parsed.
 * \param name_buf    Detected name, must be at least CPU_INFO_BUFSIZE long
 * \param speed_buf   Detected speed, must be at least CPU_INFO_BUFSIZE long
 * \param vendor_buf  Detected vendor, must be at least CPU_INFO_BUFSIZE long
 */
    void
get_cpu_linux_sparc_model_vendor(modelstr, name_buf, speed_buf, vendor_buf)
    char *modelstr;
    char *name_buf;
    char *speed_buf;
    char *vendor_buf;
{
    /* This is tricky... */
    if (0 == strncmp(modelstr, "TI ", 3))
    {
        strncpynull(vendor_buf, "Texas Instruments", CPU_INFO_BUFSIZE);
        strncpynull(name_buf, modelstr + 3, CPU_INFO_BUFSIZE);
    }
    else if (0 == strncmp(modelstr, "Texas Instruments, Inc. - ", 26))
    {
        strncpynull(vendor_buf, "Texas Instruments", CPU_INFO_BUFSIZE);
        strncpynull(name_buf, modelstr + 26, CPU_INFO_BUFSIZE);
    }
    else if (0 == strncmp(modelstr, "ROSS ", 5))
    {
        strncpynull(vendor_buf, "Ross or Bridgepoint", CPU_INFO_BUFSIZE);
        strncpynull(name_buf, modelstr + 5, CPU_INFO_BUFSIZE);
    }
    else if (0 == strncmp(modelstr, "Fujitsu or Weitek ", 18))
    {
        strncpynull(vendor_buf, "Fujitsu or Weitek", CPU_INFO_BUFSIZE);
        strncpynull(name_buf, modelstr + 18, CPU_INFO_BUFSIZE);
    }
    else if (0 == strncmp(modelstr, "Fujitsu ", 8))
    {
        /* possibly extra whitespace */
        char *name = modelstr + 8;
        while (*name == ' ')
            name++;
        strncpynull(vendor_buf, "Fujitsu", CPU_INFO_BUFSIZE);
        strncpynull(name_buf, name, CPU_INFO_BUFSIZE);
    }
    else
        strncpynull(name_buf, modelstr, CPU_INFO_BUFSIZE);
}

/**
 * Get cpuinfo from a Linux/SPARC cpuinfo file.
 *
 * Parses a /proc/cpuinfo file (must be already fopen()ed) and places the
 * results into cpu_info.
 *
 * \param cpu_info     Parsed info goes here
 * \param proccpuinfo  Filehandle (pre-fopen()ed) from which to read
 */
    int
get_cpu_linux_sparc_info(cpu_info, proccpuinfo)
    cpu_info_T **cpu_info;
    FILE *proccpuinfo;
{
    char keybuf[KEYBUF_SIZE];
    char valuebuf[VALUEBUF_SIZE];
    one_cpu_info_T ref_cpu;
    int i, hz = 0;

    /* for sparc, we grab the "ncpus active" entry to find out how many CPUs we
     * have, and then duplicate from our reference CPU at the end. In case we
     * don't detect the count, assume 1. */
    (*cpu_info)->count = 1;

    strncpynull(ref_cpu.vendor,        "", CPU_INFO_BUFSIZE);
    strncpynull(ref_cpu.model_name,    "", CPU_INFO_BUFSIZE);
    strncpynull(ref_cpu.model_speed,   "", CPU_INFO_BUFSIZE);

    while (!feof(proccpuinfo))
    {
        if (get_proc_cpuinfo_entry(proccpuinfo, keybuf, valuebuf))
        {
            if (0 == strcmp(keybuf, "cpu"))
                get_cpu_linux_sparc_model_vendor(valuebuf,
                        ref_cpu.model_name,
                        ref_cpu.model_speed,
                        ref_cpu.vendor);
            else if (0 == strcmp(keybuf, "ncpus active"))
            {

                (*cpu_info)->count = strtol(valuebuf, NULL, 10);
            }
            else if (0 == strncmp(keybuf, "Cpu", 3))
                if ((0 == strcmp(keybuf + 4, "ClkTck")) ||
                        (0 == strcmp(keybuf + 5, "ClkTck")))
                {
                    int new_hz = strtol(valuebuf, NULL, 16);
                    if (new_hz > hz)
                        hz = new_hz;
                }
        }
    }

    if (hz > 0)
        snprintf(ref_cpu.model_speed, CPU_INFO_BUFSIZE, "%d Hz", hz);

    *cpu_info = realloc(*cpu_info,
            sizeof(cpu_info_T) +
            (*cpu_info)->count * sizeof(one_cpu_info_T));
    for (i = 0 ; i < (*cpu_info)->count ; ++i)
        memcpy(&(*cpu_info)->processors[i], &ref_cpu, sizeof(one_cpu_info_T));

    return 1;
}

/* vim: set tw=80 sw=4 et : */
