/*
 * Copyright (c) 2004 Ciaran McCreesh <ciaranm at gentoo.org>
 *
 * This file is part of uhinv.
 *
 * uhinv is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * uhinv is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * uhinv; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * \file   src/cpu/cpu_linux_x86.c
 * Linux/x86 CPU detection.
 *
 * Includes functions for detecting and parsing Linux/x86 cpuinfo.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "src/uhinv.h"
#include "src/util/util.h"

#include "cpu_generic.h"
#include "cpu_linux.h"

/**
 * Decode a Linux/x86 model string.
 *
 * Splits modelstr into name and speed parts.
 *
 * \param modelstr    Model string to be parsed.
 * \param name_buf    Detected name, must be at least CPU_INFO_BUFSIZE long
 * \param speed_buf   Detected speed, must be at least CPU_INFO_BUFSIZE long
 */
    void
get_cpu_linux_x86_model(modelstr, name_buf, speed_buf)
    char *modelstr;
    char *name_buf;
    char *speed_buf;
{
    /* If the speed is in the model string, it's generally the last token, and
     * it starts with a digit. However, the original Pentiums give a range in
     * the form '75 - 200', so we need to scan backwards. If we end up with just
     * a single digit, we don't want it (eg VIA Samuel 2). Some Athlon XPs use
     * XP2400+ with no space. */
    char *spacep = strrchr(modelstr, ' ');
    if ((spacep) && (*(spacep + 1) == 'X') && (*(spacep + 2) == 'P'))
    {
        /* hack in an extra space */
        char *cp;
        for (cp = modelstr + strlen(modelstr) ; cp >= (spacep + 2) ; --cp)
            *(cp + 1) = *cp;
        *(spacep + 3) = ' ';
        spacep = strrchr(modelstr, ' ');
    }

    if (spacep && (*(spacep + 1) >= '0') && (*(spacep + 1) <= '9') &&
            (*(spacep + 2) != '\0'))
    {
        if ((spacep - 1 > modelstr) && (*(spacep - 1) == '-'))
        {
            spacep -= 2;
            while (--spacep >= modelstr)
                if (*spacep == ' ')
                    break;
            if (spacep > modelstr)
                strncpynull(speed_buf, spacep + 1, CPU_INFO_BUFSIZE);
            else
                spacep = modelstr + strlen(modelstr);

        }
        else
            strncpynull(speed_buf, spacep + 1, CPU_INFO_BUFSIZE);
    }
    else
        spacep = modelstr + strlen(modelstr);
    *spacep = '\0';

    strncpynull(name_buf, modelstr, CPU_INFO_BUFSIZE);
}

/**
 * Get cpuinfo from a Linux/x86 cpuinfo file.
 *
 * Parses a /proc/cpuinfo file (must be already fopen()ed) and places the
 * results into cpu_info.
 *
 * \param cpu_info     Parsed info goes here
 * \param proccpuinfo  Filehandle (pre-fopen()ed) from which to read
 */
    int
get_cpu_linux_x86_info(cpu_info, proccpuinfo)
    cpu_info_T **cpu_info;
    FILE *proccpuinfo;
{
    char keybuf[KEYBUF_SIZE];
    char valuebuf[VALUEBUF_SIZE];
    one_cpu_info_T dummy_cpu;

    (*cpu_info)->count = 0;

    one_cpu_info_T *curr_cpu = &dummy_cpu;

    while (!feof(proccpuinfo))
    {
        if (get_proc_cpuinfo_entry(proccpuinfo, keybuf, valuebuf))
        {
            if (0 == strcmp(keybuf, "processor"))
            {
                (*cpu_info)->count++;
                *cpu_info = realloc(*cpu_info,
                        sizeof(cpu_info_T) +
                        (*cpu_info)->count * sizeof(one_cpu_info_T));
                curr_cpu = &(*cpu_info)->processors[(*cpu_info)->count - 1];

                strncpynull(curr_cpu->vendor,        "", CPU_INFO_BUFSIZE);
                strncpynull(curr_cpu->model_name,    "", CPU_INFO_BUFSIZE);
                strncpynull(curr_cpu->model_speed,   "", CPU_INFO_BUFSIZE);
            }
            else if (0 == strcmp(keybuf, "vendor_id"))
            {
                strncpynull(curr_cpu->vendor,
                        valuebuf, CPU_INFO_BUFSIZE);
            }
            else if (0 == strcmp(keybuf, "model name"))
                get_cpu_linux_x86_model(valuebuf,
                        curr_cpu->model_name,
                        curr_cpu->model_speed);
            else if (0 == strcmp(keybuf, "cpu MHz"))
            {
                if (!*curr_cpu->model_speed)
                    strncpynull(curr_cpu->model_speed, valuebuf,
                            CPU_INFO_BUFSIZE);
            }
        }
    }

    return 1;
}

/* vim: set tw=80 sw=4 et : */
