/*
 * Copyright (c) 2004 Ciaran McCreesh <ciaranm at gentoo.org>
 *
 * This file is part of uhinv.
 *
 * uhinv is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * uhinv is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * uhinv; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * \file   src/cpu/cpu_linux_x86_64.c
 * Linux/x86_64 CPU detection.
 *
 * Includes functions for detecting and parsing Linux/x86_64 cpuinfo.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "src/uhinv.h"
#include "src/util/util.h"

#include "cpu_generic.h"
#include "cpu_linux.h"

/**
 * Decode a Linux/x86_64 model string.
 *
 * Splits modelstr into name and speed parts.
 *
 * \param modelstr    Model string to be parsed.
 * \param name_buf    Detected name, must be at least CPU_INFO_BUFSIZE long
 * \param speed_buf   Detected speed, must be at least CPU_INFO_BUFSIZE long
 */
    void
get_cpu_linux_x86_64_model(modelstr, name_buf, speed_buf)
    char *modelstr;
    char *name_buf;
    char *speed_buf;
{
    /* The last token is the performance indicator. */
    char *spacep = strrchr(modelstr, ' ');
    if (spacep)
    {
        strncpynull(speed_buf, spacep + 1, CPU_INFO_BUFSIZE);
    }
    else
        spacep = modelstr + strlen(modelstr);
    *spacep = '\0';

    strncpynull(name_buf, modelstr, CPU_INFO_BUFSIZE);
}


/**
 * Get cpuinfo from a Linux/x86_64 cpuinfo file.
 *
 * Parses a /proc/cpuinfo file (must be already fopen()ed) and places the
 * results into cpu_info.
 *
 * \param cpu_info     Parsed info goes here
 * \param proccpuinfo  Filehandle (pre-fopen()ed) from which to read
 */
    int
get_cpu_linux_x86_64_info(cpu_info, proccpuinfo)
    cpu_info_T **cpu_info;
    FILE *proccpuinfo;
{
    char keybuf[KEYBUF_SIZE];
    char valuebuf[VALUEBUF_SIZE];
    one_cpu_info_T dummy_cpu;

    (*cpu_info)->count = 0;

    one_cpu_info_T *curr_cpu = &dummy_cpu;

    while (!feof(proccpuinfo))
    {
        if (get_proc_cpuinfo_entry(proccpuinfo, keybuf, valuebuf))
        {
            if (0 == strcmp(keybuf, "processor"))
            {
                (*cpu_info)->count++;
                *cpu_info = realloc(*cpu_info,
                        sizeof(cpu_info_T) +
                        (*cpu_info)->count * sizeof(one_cpu_info_T));
                curr_cpu = &(*cpu_info)->processors[(*cpu_info)->count - 1];

                strncpynull(curr_cpu->vendor,        "", CPU_INFO_BUFSIZE);
                strncpynull(curr_cpu->model_name,    "", CPU_INFO_BUFSIZE);
                strncpynull(curr_cpu->model_speed,   "", CPU_INFO_BUFSIZE);
            }
            else if (0 == strcmp(keybuf, "vendor_id"))
            {
                if (arguments.verbose > 1)
                    printf(">>> Found vendor_id \"%s\"\n", valuebuf);
                strncpynull(curr_cpu->vendor,
                        valuebuf, CPU_INFO_BUFSIZE);
            }
            else if (0 == strcmp(keybuf, "model name"))
                get_cpu_linux_x86_64_model(valuebuf,
                        curr_cpu->model_name,
                        curr_cpu->model_speed);
        }
    }

    return 1;
}

/* vim: set tw=80 sw=4 et : */
