/*
 * Copyright (c) 2004 Ciaran McCreesh <ciaranm at gentoo.org>
 *
 * This file is part of uhinv.
 *
 * uhinv is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * uhinv is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * uhinv; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * \file   src/devices/devices_linux_pci.c
 * Handle Linux PCI devices.
 *
 * Handles Linux PCI devices via /sys/bus/pci and the pci.ids database. This
 * code is ugly, messy and needs a serious kick in the face.
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>

#include "src/uhinv.h"
#include "src/util/util.h"
#include "src/util/locations.h"

#include "devices_generic.h"
#include "devices_linux.h"

#define PCI_VENDOR_ID_SIZE       31
#define PCI_VENDOR_NAME_SIZE     31

#define PCI_DEVICE_ID_SIZE       31
#define PCI_DEVICE_NAME_SIZE     127

/** One PCI database entry. */
typedef struct
{
    char vendor_id[PCI_VENDOR_ID_SIZE];
    char vendor_name[PCI_VENDOR_NAME_SIZE];
    char device_id[PCI_DEVICE_ID_SIZE];
    char device_name[PCI_DEVICE_NAME_SIZE];
} one_pci_db_entry_T;

/** A PCI database. */
typedef struct
{
    int count;
    int size;
    one_pci_db_entry_T entries[];
} pci_db_T;

/**
 * Add a PCI database entry to the database.
 *
 * \param db            The database
 * \param vendor_id     The vendor id for the new device
 * \param vendor_name   The vendor name for the new device
 * \param device_id     The device id for the new device
 * \param device_name   The device name for the new device
 */
    void
add_pci_id(db, vendor_id, vendor_name, device_id, device_name)
    pci_db_T **db;
    char *vendor_id;
    char *vendor_name;
    char *device_id;
    char *device_name;
{
    (*db)->count++;
    if ((*db)->count > (*db)->size)
    {
        (*db)->size = 1 + 2 * (*db)->size;
        *db = realloc(*db, sizeof(pci_db_T) +
                ((*db)->size * sizeof(one_pci_db_entry_T)));
    }
    strncpynull((*db)->entries[(*db)->count - 1].vendor_id, vendor_id,
            PCI_VENDOR_ID_SIZE);
    strncpynull((*db)->entries[(*db)->count - 1].vendor_name, vendor_name,
            PCI_VENDOR_NAME_SIZE);
    strncpynull((*db)->entries[(*db)->count - 1].device_id, device_id,
            PCI_DEVICE_ID_SIZE);
    strncpynull((*db)->entries[(*db)->count - 1].device_name, device_name,
            PCI_DEVICE_NAME_SIZE);
    kill_training_whitespace((*db)->entries[(*db)->count - 1].vendor_name);
    kill_training_whitespace((*db)->entries[(*db)->count - 1].device_name);
}

/**
 * Add PCI IDs to a database from a file.
 *
 * \param filename   The filename to read
 * \param db         The database to add to
 */
    void
add_pci_ids(filename, db)
    char *filename;
    pci_db_T **db;
{
    FILE *fh;
    if ((fh = fopen(filename, "r")))
    {
        char buf[255];
        while (!feof(fh))
            if (fgets(buf, 255, fh))
            {
                char vendor_id[PCI_VENDOR_ID_SIZE];
                char vendor_name[PCI_VENDOR_NAME_SIZE];

                if ((buf[0] >= '0' && buf[0] <= '9') ||
                        (buf[0] >= 'a' && buf[0] <= 'f'))
                {
                    char *delimp = strstr(buf, "  ");
                    if (delimp)
                    {
                        *delimp = '\0';
                        strncpynull(vendor_id, buf, PCI_VENDOR_ID_SIZE);
                        strncpynull(vendor_name, delimp + 2, PCI_VENDOR_NAME_SIZE);
                        add_pci_id(db, vendor_id, vendor_name, "", "");
                    }
                }
                else if (buf[0] == '\t' && buf[1] != '\t')
                {
                    char device_id[PCI_DEVICE_ID_SIZE];
                    char device_name[PCI_DEVICE_NAME_SIZE];
                    char *delimp = strstr(buf, "  ");
                    if (delimp)
                    {
                        *delimp = '\0';
                        strncpynull(device_id, buf + 1, PCI_DEVICE_ID_SIZE);
                        strncpynull(device_name, delimp + 2, PCI_DEVICE_NAME_SIZE);
                        add_pci_id(db, vendor_id, vendor_name, device_id, device_name);
                    }
                }
                else if (buf[0] == 'C')
                {
                    strncpynull(vendor_id, buf + 2, 3);
                    strncpynull(vendor_name, buf + 6, PCI_VENDOR_NAME_SIZE);
                    add_pci_id(db, vendor_id, vendor_name, "", "");
                }
            }
        fclose(fh);
    }
    else
        fprintf(stderr, "Error opening \"%s\"\n", filename);
}

/**
 * Get the vendor and device names for a vendor id / device id pair.
 *
 * \param db           A PCI database
 * \param vendor_id    Vendor ID to search for
 * \param vendor_name  Vendor name goes here
 * \param device_id    Device ID to search for
 * \param device_name  Device name goes here
 * \return             Whether there was a match
 */
    int
decode_pci_device(db, vendor_id, vendor_name, device_id, device_name)
    pci_db_T *db;
    char *vendor_id;
    char *vendor_name;
    char *device_id;
    char *device_name;
{
    /* TODO: binary search */
    int e_id;
    for (e_id = 0 ; e_id < db->count ; ++e_id)
        if (0 == strcmp(db->entries[e_id].vendor_id, vendor_id))
        {
            strncpynull(vendor_name, db->entries[e_id].vendor_name,
                    PCI_VENDOR_NAME_SIZE);
            strncpynull(device_name, "", PCI_DEVICE_NAME_SIZE);
            for ( ; e_id < db->count ; ++e_id)
            {
                if (0 == strcmp(db->entries[e_id].device_id, device_id))
                {
                    strncpynull(device_name, db->entries[e_id].device_name,
                            PCI_DEVICE_NAME_SIZE);
                    break;
                }
            }
            return 1;
        }
    return 0;
}

/**
 * Detect and add Linux PCI devices to a device list.
 *
 * \param devinfo    The device list
 */
    void
add_linux_pci_devices(devinfo)
    devices_info_T **devinfo;
{
    DIR *dirh;
    struct dirent *dent;

    if ((dirh = opendir("/sys/bus/pci/devices")))
    {
        pci_db_T *db = malloc(sizeof(pci_db_T));
        db->count = 0;
        db->size = 0;
        add_pci_ids(LOCATION_PKGDATADIR "/pci.ids", &db);

        while ((dent = readdir(dirh)))
        {
            if ('.' == dent->d_name[0])
                continue;

            (*devinfo)->count++;
            *devinfo = realloc(*devinfo,
                    sizeof(devices_info_T) +
                    (*devinfo)->count * sizeof(one_device_info_T));
            one_device_info_T *thisdev = &(*devinfo)->devices[(*devinfo)->count - 1];
            strncpynull(thisdev->bus, "PCI", DEVICE_INFO_BUFSIZE);

            /* location */
            strncpynull(thisdev->location, dent->d_name, DEVICE_INFO_BUFSIZE);

            /* vendor, name, class */
            strncpynull(thisdev->vendor, "Unknown", DEVICE_INFO_BUFSIZE);
            strncpynull(thisdev->name, "Unknown", DEVICE_INFO_BUFSIZE);
            strncpynull(thisdev->devclass, "Unknown", DEVICE_INFO_BUFSIZE);
            {
                char vendor_id[PCI_VENDOR_ID_SIZE];
                char vendor_name[PCI_VENDOR_NAME_SIZE];
                char device_id[PCI_DEVICE_ID_SIZE];
                char device_name[PCI_DEVICE_NAME_SIZE];

                FILE *fh;
                int fn_sz = (strlen(dent->d_name) +
                        strlen("/sys/bus/pci/devices") + 12);
                char *fn = (char *)malloc(sizeof(char) * fn_sz);
                snprintf(fn, fn_sz, "%s/%s/%s", "/sys/bus/pci/devices",
                        dent->d_name, "vendor");
                if ((fh = fopen(fn, "r")))
                {
                    char *vendor_id_p = vendor_id;
                    fgets(vendor_id, PCI_VENDOR_ID_SIZE, fh);
                    kill_training_whitespace(vendor_id);
                    if (0 == strncmp(vendor_id_p, "0x", 2))
                        vendor_id_p += 2;

                    fclose(fh);

                    snprintf(fn, fn_sz, "%s/%s/%s", "/sys/bus/pci/devices",
                            dent->d_name, "device");
                    if ((fh = fopen(fn, "r")))
                    {
                        char *device_id_p = device_id;
                        fgets(device_id, PCI_DEVICE_ID_SIZE, fh);
                        kill_training_whitespace(device_id);
                        if (0 == strncmp(device_id_p, "0x", 2))
                            device_id_p += 2;
                        if (decode_pci_device(db, vendor_id_p, vendor_name, device_id_p,
                                    device_name))
                        {
                            strncpynull(thisdev->vendor, vendor_name, DEVICE_INFO_BUFSIZE);
                            if (*device_name)
                                strncpynull(thisdev->name, device_name, DEVICE_INFO_BUFSIZE);
                            else
                            {
                                snprintf(thisdev->vendor, DEVICE_INFO_BUFSIZE,
                                        "Unknown PCI Vendor %s", vendor_id);
                                snprintf(thisdev->name, DEVICE_INFO_BUFSIZE,
                                        "Unknown PCI Device %s", device_id);
                            }
                        }
                        else
                        {
                            snprintf(thisdev->vendor, DEVICE_INFO_BUFSIZE,
                                    "Unknown PCI Vendor %s", vendor_id);
                            snprintf(thisdev->name, DEVICE_INFO_BUFSIZE,
                                    "Unknown PCI Device %s", device_id);
                        }

                        fclose(fh);
                    }
                }
                else
                    fprintf(stderr, "Error opening \"%s\"\n", fn);

                snprintf(fn, fn_sz, "%s/%s/%s", "/sys/bus/pci/devices",
                        dent->d_name, "class");
                if ((fh = fopen(fn, "r")))
                {
                    char n1[PCI_VENDOR_NAME_SIZE], n2[PCI_DEVICE_NAME_SIZE],
                         cbuf[DEVICE_INFO_BUFSIZE], cmaj[3], cmin[3];
                    char *class_id_p = cbuf;
                    fgets(cbuf, DEVICE_INFO_BUFSIZE, fh);
                    kill_training_whitespace(cbuf);
                    if (0 == strncmp(class_id_p, "0x", 2))
                        class_id_p += 2;
                    strncpynull(cmaj, class_id_p, 3);
                    strncpynull(cmin, class_id_p + 2, 3);
                    if (decode_pci_device(db, cmaj, n1, cmin, n2))
                        snprintf(thisdev->devclass, DEVICE_INFO_BUFSIZE, "%s (%s)",
                                n2, n1);
                    fclose(fh);
                }

                free(fn);
            }

        }
        closedir(dirh);
        free(db);
    }
}

/* vim: set tw=80 sw=4 et : */
