/*
 * uhinv (Universal Hardware Inventory)
 * src/display_uhinv.c
 * Output Information
 *
 * Copyright (c) 2004 Ciaran McCreesh <ciaranm at gentoo.org>
 *
 * This file is part of uhinv.
 *
 * uhinv is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * uhinv is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * uhinv; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 325, Boston, MA  02111-1257  USA
 */

/**
 * \file   src/display_uhinv.c
 * Display hardware information.
 *
 * Displays hardware information. Delegates for finding and doing formatting of
 * the output.
 */

#include <stdlib.h>
#include <stdio.h>

#include "src/uhinv.h"

#include "src/os/os_generic.h"
#include "src/cpu/cpu_generic.h"
#include "src/mem/mem_generic.h"
#include "src/devices/devices_generic.h"

#include "display_uhinv_pretty.h"
#include "display_uhinv_text.h"

#include "src/filters.h"

/**
 * Display a section header.
 *
 * Delegates to do the actual displaying.
 *
 * \param indent       Indent level
 * \param title        Title to display
 */
    void
display_section_header(indent, title)
    unsigned int indent;
    char *title;
{
    if (arguments.format == ARGUMENTS_FORMAT_PRETTY)
        display_section_header_pretty(indent, title);
    else
        display_section_header_text(indent, title);
}

/**
 * Display a compact information string indented 'indent' levels.
 *
 * The 'key' is always displayed ; the values are displayed with a space after
 * them if they are not empty. A single blank value does not prevent subsequent
 * values from being displayed.
 *
 * \param indent    Indent level
 * \param key       The key
 * \param value1    First value to display (can be an empty string)
 * \param value2    Second value to display (can be an empty string)
 * \param value3    Third value to display (can be an empty string)
 */
    void
display_compact_str(indent, key, value1, value2, value3)
    unsigned int indent;
    char *key;
    char *value1;
    char *value2;
    char *value3;
{
    if (arguments.format == ARGUMENTS_FORMAT_PRETTY)
        display_compact_str_pretty(indent, key, value1, value2, value3);
    else
        display_compact_str_text(indent, key, value1, value2, value3);
}

/**
 * Display a key/value pair indented 'indent' levels.
 *
 * \param indent    Indent level
 * \param key       The key to be displayed
 * \param value     The value to be displayed
 */
    void
display_kvpair_str(indent, key, value)
    unsigned int indent;
    char *key;
    char *value;
{
    if (arguments.format == ARGUMENTS_FORMAT_PRETTY)
        display_kvpair_str_pretty(indent, key, value);
    else
        display_kvpair_str_text(indent, key, value);
}

/**
 * Display a key/value pair indented 'indent' levels.
 *
 * \param indent    Indent level
 * \param key       The key to be displayed
 * \param value     The value to be displayed
 */
    void
display_kvpair_int(indent, key, value)
    unsigned int indent;
    char *key;
    int value;
{
    char buf[30];
    snprintf(buf, 30, "%d", value);
    display_kvpair_str(indent, key, buf);
}

/**
 * Display operating system information.
 *
 * \param osinfo   Operating system to display
 */
    void
display_os(osinfo)
    operating_system_info_T *osinfo;
{
    display_section_header(0, "Operating System Information");
    if (arguments.verbose > 0)
    {
        display_kvpair_str(1, "Name:",     osinfo->name);
        display_kvpair_str(1, "Release:",  osinfo->release);
        display_kvpair_str(1, "Version:",  osinfo->version);
        display_kvpair_str(1, "Machine:",  osinfo->machine);
        display_kvpair_str(1, "Hostname:", osinfo->hostname);
    }
    else
    {
        char buf3[255], buf4[255];
        snprintf(buf3, 255, "%s %s", osinfo->release, osinfo->version);
        snprintf(buf4, 255, "on %s", osinfo->machine);
        display_compact_str(1, osinfo->name, "", buf3, buf4);
        display_compact_str(1, "Hostname", "", osinfo->hostname, "");
    }
    printf("\n");
}

/**
 * Display memory information.
 *
 * \param meminfo       Memory information to display
 */
    void
display_mem(meminfo)
    mem_info_T *meminfo;
{
    display_section_header(0, "Memory Information");
    if (arguments.verbose > 0)
    {
        display_kvpair_str(1, "Total RAM:",  meminfo->mem_total);
        display_kvpair_str(1, "Total Swap:", meminfo->swap_total);

    }
    else
    {
        display_compact_str(1, "Total RAM ", "", meminfo->mem_total, "");
        display_compact_str(1, "Total Swap", "", meminfo->swap_total, "");
    }
    printf("\n");
}

/**
 * Display an error.
 *
 * \param indent     Indent level
 * \param msg        Error message to display
 */
    void
display_error_str(indent, msg)
    int indent;
    char *msg;
{
    int i;
    for (i = 0 ; i < (2 * indent) ; ++i)
        printf(" ");
    printf("%s\n", msg);
}

/**
 * Display CPU information.
 *
 * \param cpuinfo       CPU information to display
 */
    void
display_cpu(cpuinfo)
    cpu_info_T *cpuinfo;
{
    int e_cpu;
    display_section_header(0, "CPU Information");
    if (cpuinfo->count > 0)
    {
        if (arguments.verbose > 0)
        {
            display_kvpair_int(1, "Number of CPUs:", cpuinfo->count);
            printf("\n");
        }

        for (e_cpu = 0 ; e_cpu < cpuinfo->count ; ++e_cpu)
        {
            if (arguments.verbose > 0)
            {
                char buf[255];
                snprintf(buf, 255, "CPU %d Information", e_cpu + 1);
                display_section_header(1, buf);
                if (*cpuinfo->processors[e_cpu].vendor)
                    display_kvpair_str(2, "Vendor:",
                            cpuinfo->processors[e_cpu].vendor);
                display_kvpair_str(2, "Model Name:",
                        cpuinfo->processors[e_cpu].model_name);
                if (*cpuinfo->processors[e_cpu].model_speed)
                    display_kvpair_str(2, "Performance Rating:",
                            cpuinfo->processors[e_cpu].model_speed);
            }
            else
            {
                char buf1[255], buf3[255], buf4[255];
                snprintf(buf1, 255, "CPU %-2.2d", e_cpu + 1);
                snprintf(buf3, 255, "%s", cpuinfo->processors[e_cpu].vendor);
                snprintf(buf4, 255, "%s %s",
                        cpuinfo->processors[e_cpu].model_name,
                        cpuinfo->processors[e_cpu].model_speed);
                display_compact_str(1, buf1, "", buf3, buf4);
            }
        }
    }
    else
    {
        if (arguments.verbose > 0)
            display_kvpair_str(1, "Number of CPUs:", "Unknown");
        else
            display_error_str(1, "No CPUs found");
    }
    printf("\n");
}

/**
 * Display device information.
 *
 * \param devinfo       Device information to display
 */
    void
display_devices(devinfo)
    devices_info_T *devinfo;
{
    int e_dev;
    display_section_header(0, "Device Information");
    if (devinfo->count > 0)
    {
        if (arguments.filter_count > 0)
        {
            int match_count = 0;
            for (e_dev = 0 ; e_dev < devinfo->count ; ++e_dev)
                if (device_filters_match(arguments.filters, arguments.filter_count,
                            arguments.filter_mode, devinfo->devices[e_dev]))
                    ++match_count;
            if (arguments.verbose)
            {
                if (match_count)
                    display_kvpair_int(1, "Matching Devices:", match_count);
                else
                {
                    display_kvpair_str(1, "Matching Devices:", "None Found");
                    return;
                }
            }
            else
                if (!match_count)
                    display_error_str(1, "No Matching Devices");
        }
        else
            if (arguments.verbose > 0)
                display_kvpair_int(1, "Detected Devices:", devinfo->count);

        int e_dev_shown = 0;
        for (e_dev = 0 ; e_dev < devinfo->count ; ++e_dev)
        {
            if (device_filters_match(arguments.filters, arguments.filter_count,
                        arguments.filter_mode, devinfo->devices[e_dev]))
            {
                char buf[255];
                snprintf(buf, 255, "Device %d Information", ++e_dev_shown);
                if (arguments.verbose > 0)
                {
                    display_section_header(1, buf);
                    display_kvpair_str(2, "Bus:",
                            devinfo->devices[e_dev].bus);
                    display_kvpair_str(2, "Location:",
                            devinfo->devices[e_dev].location);
                    display_kvpair_str(2, "Class:",
                            devinfo->devices[e_dev].devclass);
                    if (*devinfo->devices[e_dev].vendor)
                        display_kvpair_str(2, "Vendor:",
                                devinfo->devices[e_dev].vendor);
                    display_kvpair_str(2, "Name:",
                            devinfo->devices[e_dev].name);
                }
                else
                {
                    char buf1[255], buf2[255];
                    snprintf(buf1, 255, "%s %s", devinfo->devices[e_dev].bus,
                            devinfo->devices[e_dev].location);
                    snprintf(buf2, 255, "%s:", devinfo->devices[e_dev].devclass);
                    display_compact_str(1, buf1, buf2, "", "");
                    display_compact_str(2, "", "",
                            devinfo->devices[e_dev].vendor,
                            devinfo->devices[e_dev].name);
                }
            }
        }

        printf("\n");
    }
    else
    {
        if (arguments.verbose > 0)
            display_kvpair_str(1, "Detected Devices:", "None Recognised");
        else
            display_error_str(1, "No Detected Devices");
    }
}

/**
 * Get and display hardware information.
 */
    void
display_uhinv()
{
    operating_system_info_T osinfo;
    cpu_info_T *cpuinfo = malloc(sizeof(cpu_info_T));
    mem_info_T meminfo;
    devices_info_T *devinfo = malloc(sizeof(devices_info_T));

    /* Get information. We need os info for CPU detection. */
    if ((arguments.info & ARGUMENTS_INFO_OS) ||
            (arguments.info & ARGUMENTS_INFO_CPU))
        get_operating_system_info(&osinfo);

    if (arguments.info & ARGUMENTS_INFO_CPU)
    {
        if (*arguments.arch)
            get_cpu_info(&cpuinfo, arguments.arch);
        else
            get_cpu_info(&cpuinfo, osinfo.machine);
    }

    if (arguments.info & ARGUMENTS_INFO_MEM)
        get_mem_info(&meminfo);

    if (arguments.info & ARGUMENTS_INFO_DEVICES)
        get_devices_info(&devinfo);

    /* Display information. */
    if (arguments.info & ARGUMENTS_INFO_OS)
        display_os(&osinfo);

    if (arguments.info & ARGUMENTS_INFO_MEM)
        display_mem(&meminfo);

    if (arguments.info & ARGUMENTS_INFO_CPU)
        display_cpu(cpuinfo);

    if (arguments.info & ARGUMENTS_INFO_DEVICES)
        display_devices(devinfo);

    free(cpuinfo);
}

/* vim: set tw=80 sw=4 et : */
