/*
 * Copyright (c) 2004 Ciaran McCreesh <ciaranm at gentoo.org>
 *
 * This file is part of uhinv.
 *
 * uhinv is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * uhinv is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * uhinv; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 325, Boston, MA  02111-1257  USA
 */

/**
 * \file   src/display_uhinv_pretty.c
 * Display functions: pretty format.
 *
 * Implements various display_ functions for when --format=pretty. The output
 * from these functions is ANSI-highlighted for easy reading by humans.
 */

#include <stdlib.h>
#include <stdio.h>

#include "src/uhinv.h"

#include "src/os/os_generic.h"
#include "src/cpu/cpu_generic.h"
#include "src/mem/mem_generic.h"

#define PRETTY_KEY_START    "\033[1;34m"   /**< Start of a key, blue */
#define PRETTY_KEY_END      "\033[0m"      /**< End of a key */

#define PRETTY_VALUE_START  "\033[0m"      /**< Start of a value, white */
#define PRETTY_VALUE1_START "\033[1;35m"   /**< Start of a value1, pink */
#define PRETTY_VALUE2_START "\033[0m"      /**< Start of a value2, white */
#define PRETTY_VALUE3_START "\033[1;33m"   /**< Start of a value3, yellow */
#define PRETTY_VALUE_END    "\033[0m"      /**< End of a value */

#define PRETTY_TITLE_START  "\033[1;32m"   /**< Start of a title, green */
#define PRETTY_TITLE_END    "\033[0m"      /**< End of a title */


/**
 * Display a section header with title 'title' indented 'indent' levels.
 *
 * \param indent   Indent level
 * \param title    Title to display
 */
    void
display_section_header_pretty(indent, title)
    unsigned int indent;
    char *title;
{
    int i;
    for (i = 0 ; i < (2 * indent) ; ++i)
        printf(" ");
    printf("%s%s%s\n", PRETTY_TITLE_START, title, PRETTY_TITLE_END);
}


/**
 * Display a compact information string indented 'indent' levels.
 *
 * The 'key' is always displayed ; the values are displayed with a space after
 * them if they are not empty. A single blank value does not prevent subsequent
 * values from being displayed.
 *
 * \param indent    Indent level
 * \param key       The key, coloured blue
 * \param value1    First value to display (can be an empty string), pink
 * \param value2    Second value to display (can be an empty string), white
 * \param value3    Third value to display (can be an empty string), yellow
 */
    void
display_compact_str_pretty(indent, key, value1, value2, value3)
    unsigned int indent;
    char *key;
    char *value1;
    char *value2;
    char *value3;
{
    int i;
    for (i = 0 ; i < (2 * indent) ; ++i)
        printf(" ");
    printf("%s%s%s ", PRETTY_KEY_START, key, PRETTY_KEY_END);
    if (*value1)
        printf("%s%s%s ", PRETTY_VALUE1_START, value1, PRETTY_VALUE_END);
    if (*value2)
        printf("%s%s%s ", PRETTY_VALUE2_START, value2, PRETTY_VALUE_END);
    if (*value3)
        printf("%s%s%s", PRETTY_VALUE3_START, value3, PRETTY_VALUE_END);
    printf("\n");
}


/**
 * Display a key/value pair indented 'indent' levels.
 *
 * \param indent    Indent level
 * \param key       The key to be displayed
 * \param value     The value to be displayed
 */
    void
display_kvpair_str_pretty(indent, key, value)
    unsigned int indent;
    char *key;
    char *value;
{
    int i;
    for (i = 0 ; i < (2 * indent) ; ++i)
        printf(" ");
    printf("%s%-20.20s%s", PRETTY_KEY_START, key, PRETTY_KEY_END);
    for (i = 6 - (2 * indent) ; i >= 0 ; --i)
        printf(" ");
    printf("%s%s%s\n", PRETTY_VALUE_START, value, PRETTY_VALUE_END);
}

/* vim: set tw=80 sw=4 et : */
