/*
 * Copyright (c) 2004 Ciaran McCreesh <ciaranm at gentoo.org>
 *
 * This file is part of uhinv.
 *
 * uhinv is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * uhinv is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * uhinv; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 325, Boston, MA  02111-1257  USA
 */

/**
 * \file   src/filters.c
 * Output filter functions.
 *
 * Includes functions for determining whether a filter pattern matches a string and
 * whether a set of filters match a device.
 */

#include <string.h>
#include "src/uhinv.h"
#include "src/devices/devices_generic.h"
#include "src/filters.h"

/**
 * Return whether pattern matches anywhere in text.
 *
 * Right now this is just a simple case insensitive naive matcher. TODO: use a
 * faster algorithm, allow wildcards and handle synonyms (eg CPU <-> Processor).
 *
 * \param pattern The pattern to search for
 * \param text    The text in which to search
 * \return        Whether there was a match
 */
    int
pattern_match(pattern, text)
    char *pattern;
    char *text;
{
    int pattern_l = strlen(pattern);
    while (*text)
    {
        if (0 == strncasecmp(pattern, text, pattern_l))
            return 1;
        else
            ++text;
    }
    return 0;
}


/**
 * Return whether the set of filters match device dev.
 *
 * The mode argument controls whether all, any or none of the filters must match
 * for it to be a success.
 *
 * \param filters Filters to try
 * \param count   Number of filters
 * \param mode    Match mode (FILTER_MODE_*)
 * \param dev     Device to test
 * \return        Whether the filters matched successfully
 */
    int
device_filters_match(filters, count, mode, dev)
    filter_T *filters;
    int count;
    int mode;
    one_device_info_T dev;
{
    if (count == 0)
        return 1;

    /* apply each filter in turn, keep count of how many match. */
    int e_filter, match_count = 0;
    for (e_filter = 0 ; e_filter < count ; ++e_filter)
    {
        char *pat = filters[e_filter].pattern;
        switch (filters[e_filter].kind)
        {
            case FILTER_KIND_BUS:
                if (pattern_match(pat, dev.bus))
                    ++match_count;
                break;

            case FILTER_KIND_LOCATION:
                if (pattern_match(pat, dev.location))
                    ++match_count;
                break;

            case FILTER_KIND_VENDOR:
                if (pattern_match(pat, dev.vendor))
                    ++match_count;
                break;

            case FILTER_KIND_CLASS:
                if (pattern_match(pat, dev.devclass))
                    ++match_count;
                break;

            case FILTER_KIND_NAME:
                if (pattern_match(pat, dev.name))
                    ++match_count;
                break;

            /* The 'ANY' filter is a success if it matches any of the device's
             * fields. */
            case FILTER_KIND_ANY:
                if ((pattern_match(pat, dev.bus)) ||
                        (pattern_match(pat, dev.location)) ||
                        (pattern_match(pat, dev.vendor)) ||
                        (pattern_match(pat, dev.devclass)) ||
                        (pattern_match(pat, dev.name)))
                    ++match_count;
                break;

            default:
                ;
        }

        /* optimisation: break out of the loop early if possible */
        if ((match_count > 0) &&
                ((mode == FILTER_MODE_NONE) ||
                 (mode == FILTER_MODE_ANY)))
            break;
    }

    /* We need exactly 0 matches for NONE, at least one match for ANY and count
     * matches for ALL. */
    if (mode == FILTER_MODE_NONE)
        return (match_count == 0);
    else if (mode == FILTER_MODE_ANY)
        return (match_count > 0);
    else if (mode == FILTER_MODE_ALL)
        return (match_count >= count);
    return 0;
}

/* vim: set tw=80 sw=4 et : */
