/*
 * Copyright (c) 2004 Ciaran McCreesh <ciaranm at gentoo.org>
 *
 * This file is part of uhinv.
 *
 * uhinv is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * uhinv is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * uhinv; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * \file   src/mem/mem_linux.c
 * Memory handling: Linux.
 *
 * Functions for parsing a Linux-style /proc/meminfo file.
 */

#include <string.h>
#include <stdio.h>

#include "src/uhinv.h"
#include "src/util/util.h"

#include "mem_generic.h"
#include "mem_linux.h"

#define KEYBUF_SIZE   31       /**< size of the key buffer */
#define VALUEBUF_SIZE 31       /**< size of the value buffer */
#define LINEBUF_SIZE  255      /**< size of the line buffer */

/**
 * Get the next key: value pair from a meminfo file.
 *
 * The fh argument should be fopen()ed already. There must be at least
 * KEYBUF_SIZE characters available for writing in keybuf and VALUEBUF_SIZE
 * characters available for writing in valuebuf. Returns whether a recognisable
 * line was read. A failure doesn't necessarily indicate EOF -- it could just be
 * a line in a format we don't understand.
 *
 * A meminfo file contains lines like:
 * FirstKey:         FirstValue Units
 * SecondKey:       SecondValue Units
 *
 * \param fh        Filehandle for the meminfo file. Must be fopen()ed already.
 * \param keybuf    Buffer (at least KEYBUF_SIZE big) for the parsed key.
 * \param valuebuf  Buffer (at least VALUEBUF_SIZE big) for the parsed value.
 * \return          Whether a recognisable line was read.
 */
    int
get_proc_meminfo_entry(fh, keybuf, valuebuf)
    FILE *fh;
    char *keybuf;
    char *valuebuf;
{
    char buf[LINEBUF_SIZE];
    char *delimpos;

    if (!fgets(buf, LINEBUF_SIZE, fh))
        return 0;

    /* Hunt for the ':'. If we don't find it, we've probably got some kind of
     * fancy line which we don't want to handle. */
    delimpos = strchr(buf, ':');
    if (!delimpos)
        return 0;

    /* Split the string in two. */
    *delimpos = '\0';

    /* Copy everything before the : into keybuf */
    strncpynull(keybuf, buf, KEYBUF_SIZE);

    /* Skip over any whitespace after the : */
    while (*++delimpos == ' ')
        ;

    /* Copy the value into valuebuf */
    strncpynull(valuebuf, delimpos, VALUEBUF_SIZE);

    /* Nuke the newline character */
    valuebuf[strlen(valuebuf) - 1] = '\0';

    return 1;
}

/**
 * Try to get memory information from a Linux-style /proc/meminfo file.
 *
 * Places the results into argument meminfo. If nothing can be found, meminfo is
 * left alone.
 *
 * \param  meminfo   Where to place the parsed meminfo
 */
    void
get_linux_mem_info(meminfo)
    mem_info_T *meminfo;
{
    FILE *fh;
    char keybuf[KEYBUF_SIZE];
    char valuebuf[VALUEBUF_SIZE];

    /* Open meminfo for read. TODO: hardcoded filenames are evil. */
    if (!(fh = fopen("/proc/meminfo", "r")))
    {
        perror("Error opening /proc/meminfo for read");
        return;
    }

    /* Iterate over the lines in our meminfo file */
    while (!feof(fh))
    {
        if (get_proc_meminfo_entry(fh, keybuf, valuebuf))
        {
            /* We've got a valid key: value set. */
            if (0 == strcmp(keybuf, "MemTotal"))
                strncpynull(meminfo->mem_total, valuebuf, MEM_INFO_BUFSIZE);
            else if (0 == strcmp(keybuf, "SwapTotal"))
                strncpynull(meminfo->swap_total, valuebuf, MEM_INFO_BUFSIZE);
        }
    }

    fclose(fh);
}

/* vim: set tw=80 sw=4 et : */
