/*
 * Copyright (c) 2004 Ciaran McCreesh <ciaranm at gentoo.org>
 *
 * This file is part of uhinv.
 *
 * uhinv is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * uhinv is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * uhinv; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 325, Boston, MA  02111-1257  USA
 */

/**
 * \file   src/uhinv.c
 * Main Program.
 *
 * Main program. Handles the commandline and calls the relevant display
 * function.
 */

#include <argp.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "uhinv.h"
#include "util/util.h"
#include "display_uhinv.h"
#include "config.h"

const char *argp_program_version =
        PACKAGE "-" VERSION;                     /**< program version */
const char *argp_program_bug_address =
        "<ciaranm@gentoo.org>";                  /**< program bug address */
static char doc[] =
        "uhinv -- Universal Hardware Inventory"; /**< program description */

/* Commandline options we understand */
static struct argp_option options[] =
{
    {0,0,0,0,"General Options:"},
    {"verbose",                 'v', 0,        0,
        "Produce verbose output" },


    {0,0,0,0,"Display Options:"},
    {"all-info",                'a', 0,        0,
        "Display all available information (default)" },
    {"os-info",                 'o', 0,        0,
        "Display operating system information" },
    {"cpu-info",                'c', 0,        0,
        "Display processor information" },
    {"mem-info",                'm', 0,        0,
        "Display memory and swap information" },
    {"device-info",             'd', 0,        0,
        "Display device information" },

    {0,0,0,0,"Device Filters (imply --device-info):"},
    {"filter",                  'A', "STR",    0,
        "Filter on anything" },
    {"filter-bus",              'B', "STR",    0,
        "Filter on bus (eg PCI)" },
    {"filter-class",            'C', "STR",    0,
        "Filter on class (eg VGA)" },
    {"filter-location",         'L', "STR",    0,
        "Filter on location (eg 0000:02:08.0)" },
    {"filter-vendor",           'V', "STR",    0,
        "Filter on vendor (eg IBM)" },
    {"filter-name",             'N', "STR",    0,
        "Filter on name (eg Ethernet)" },

    {0,0,0,0,"Device Filter Options"},
    {"filter-mode",             'M', "STR",    0,
        "Filter mode (ANY, ALL or NONE)" },

    {0,0,0,0,"Output Format Options:"},
    {"format",                  'f', "FORMAT", 0,
        "Select output format (text, pretty)" },

    {0,0,0,0,"Debug / Testing Options:"},
    {"force-cpuinfo",             1, "FILE",   0,
        "Use FILE instead of /proc/cpuinfo" },
    {"force-arch",                2, "ARCH",   0,
        "Use ARCH instead of auto-detecting" },

    { 0 }
};

/**
 * Parse one option
 */
    static error_t
parse_opt(key, arg, state)
    int key;
    char *arg;
    struct argp_state *state;
{
    struct arguments *arguments = state->input;

    switch (key)
    {
        case 'v': /* verbose */
            arguments->verbose++;
            break;

        case 'a': /* all info */
            arguments->info |= ARGUMENTS_INFO_ALL;
            break;

        case 'o': /* os info */
            arguments->info |= ARGUMENTS_INFO_OS;
            break;

        case 'c': /* cpu info */
            arguments->info |= ARGUMENTS_INFO_CPU;
            break;

        case 'm': /* mem info */
            arguments->info |= ARGUMENTS_INFO_MEM;
            break;

        case 'd': /* device info */
            arguments->info |= ARGUMENTS_INFO_DEVICES;
            break;

        case 'f': /* format */
            if (0 == strcasecmp(arg, "text"))
                arguments->format = ARGUMENTS_FORMAT_TEXT;
            else if (0 == strcasecmp(arg, "pretty"))
                arguments->format = ARGUMENTS_FORMAT_PRETTY;
            else
            {
                fprintf(stderr, "Unknown format \"%s\"\n", arg);
                argp_usage(state);
                exit(EXIT_FAILURE);
            }
            break;

        case 'A':
            arguments->info |= ARGUMENTS_INFO_DEVICES;
            arguments->filters[arguments->filter_count].kind = FILTER_KIND_ANY;
            arguments->filters[arguments->filter_count].pattern = arg;
            if (++(arguments->filter_count) >= FILTER_COUNT)
            {
                fprintf(stderr, "Too many filters\n");
                exit(EXIT_FAILURE);
            }
            break;

        case 'B':
            arguments->info |= ARGUMENTS_INFO_DEVICES;
            arguments->filters[arguments->filter_count].kind = FILTER_KIND_BUS;
            arguments->filters[arguments->filter_count].pattern = arg;
            if (++(arguments->filter_count) >= FILTER_COUNT)
            {
                fprintf(stderr, "Too many filters\n");
                exit(EXIT_FAILURE);
            }
            break;

        case 'C':
            arguments->info |= ARGUMENTS_INFO_DEVICES;
            arguments->filters[arguments->filter_count].kind = FILTER_KIND_CLASS;
            arguments->filters[arguments->filter_count].pattern = arg;
            if (++(arguments->filter_count) >= FILTER_COUNT)
            {
                fprintf(stderr, "Too many filters\n");
                exit(EXIT_FAILURE);
            }
            break;

        case 'L':
            arguments->info |= ARGUMENTS_INFO_DEVICES;
            arguments->filters[arguments->filter_count].kind =
                FILTER_KIND_LOCATION;
            arguments->filters[arguments->filter_count].pattern = arg;
            if (++(arguments->filter_count) >= FILTER_COUNT)
            {
                fprintf(stderr, "Too many filters\n");
                exit(EXIT_FAILURE);
            }
            break;

        case 'N':
            arguments->info |= ARGUMENTS_INFO_DEVICES;
            arguments->filters[arguments->filter_count].kind =
                FILTER_KIND_NAME;
            arguments->filters[arguments->filter_count].pattern = arg;
            if (++(arguments->filter_count) >= FILTER_COUNT)
            {
                fprintf(stderr, "Too many filters\n");
                exit(EXIT_FAILURE);
            }
            break;

        case 'V':
            arguments->info |= ARGUMENTS_INFO_DEVICES;
            arguments->filters[arguments->filter_count].kind =
                FILTER_KIND_VENDOR;
            arguments->filters[arguments->filter_count].pattern = arg;
            if (++(arguments->filter_count) >= FILTER_COUNT)
            {
                fprintf(stderr, "Too many filters\n");
                exit(EXIT_FAILURE);
            }
            break;

        case 'M':
            if (0 == strcasecmp(arg, "any"))
                arguments->filter_mode = FILTER_MODE_ANY;
            else if (0 == strcasecmp(arg, "all"))
                arguments->filter_mode = FILTER_MODE_ALL;
            else if (0 == strcasecmp(arg, "none"))
                arguments->filter_mode = FILTER_MODE_NONE;
            else
            {
                fprintf(stderr, "filter-mode should be one of ANY, ALL or NONE\n");
                exit(EXIT_FAILURE);
            }
            break;

        case 1: /* /proc/cpuinfo file */
            strncpynull(arguments->proc_cpuinfo, arg, ARGUMENTS_BUFSIZE);
            break;

        case 2: /* override arch */
            strncpynull(arguments->arch, arg, ARGUMENTS_BUFSIZE);
            break;

        case ARGP_KEY_ARG:
            argp_usage(state);
            return EINVAL;

        case ARGP_KEY_END:
            break;

        default:
            return ARGP_ERR_UNKNOWN;
    }
    return 0;
}

static struct argp argp = { options, parse_opt, 0, doc };

struct arguments arguments;

/**
 * Main function
 */
    int
main(argc, argv)
    int argc;
    char **argv;
{
    arguments.verbose      = 0;
    arguments.info         = 0;
    arguments.format       = 0;
    arguments.filter_count = 0;
    arguments.filter_mode  = FILTER_MODE_ANY;
    strncpynull(arguments.proc_cpuinfo, "/proc/cpuinfo", ARGUMENTS_BUFSIZE);

    argp_parse(&argp, argc, argv, 0, 0, &arguments);
    if (0 == arguments.info)
        arguments.info = ARGUMENTS_INFO_ALL;
    if (0 == arguments.format)
    {
        if (isatty(1))
            arguments.format = ARGUMENTS_FORMAT_PRETTY;
        else
            arguments.format = ARGUMENTS_FORMAT_TEXT;
    }

    display_uhinv();
    return EXIT_SUCCESS;
}

/* vim: set tw=80 sw=4 et : */
